/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import java.util.HashMap;
import java.util.Iterator;

import org.w3c.dom.*;


/**
 * Children:<br>
 *   error<br>
 *   format<br>
 */
public abstract class A_Validator implements I_Validator {

	protected Form form;
	private HashMap errorMessages;
	private String tagName;
	
	public A_Validator(Form form) {
		super();
	}
	
	public A_Validator(Form form, Node validator) {
		super();
		this.form = form;
		populateFromNode(validator);
	}
 
	public String tagName() {
		return tagName;
	}

	public void populateFromNode(Node validator) 
	{
		this.tagName = validator.getNodeName();
		/** No attributes for now 
		NamedNodeMap attributes = field.getAttributes();
		int len = attributes.getLength();
		for(int i = 0; i < len; i++) {
			Node attr = attributes.item(i);
			short at = attr.getNodeType();
			String an = attr.getNodeName();
			handleAttribute(attr, at, an);
		}
		**/
		NodeList children = validator.getChildNodes();
		int len = children.getLength();
		for(int i = 0; i < len; i++) {
			Node child = children.item(i);
			short ct = child.getNodeType();
			String cn = child.getNodeName();
			handleChildNode(child, ct, cn);
		}
	}
	
	public boolean handleChildNode(Node child, short childType, String childName)
	{
		if(childType == Node.ELEMENT_NODE) {
			if(childName.equals(Form.FBTAG_ERROR)) {
				String v = Form.collectText(child);
				String l = "";
				NamedNodeMap attrs = child.getAttributes();
				for(int ii=0; ii<attrs.getLength(); ii++) {
					Node attr = attrs.item(ii);
					String an = attr.getNodeName();
					if(an.equals(Form.FBATTR_LOCALE)) {
						l = Form.collectText(attr);
					}
				} 	
				setErrorMessage(v, l);
				return true;
			}
		}
		return false;
	}
	
	//
	// Accessors
	//
	
	/**
	 * First it checks the error based on the current locale of the form, if that fails
	 * it tries one more time with the "" locale.
	 * @return
	 */
	public String getErrorMessage() {
		String retval = getErrorMessage(form.getLocale());
		if(retval==null)
			retval = (String)errorMessages.get("");
		if(retval==null)
			return tagName; // just something then...
		return retval;
	}
	
	public void setErrorMessage(String str) {
		setErrorMessage(str, "");
	}

	public String getErrorMessage(String locale) {
		if(errorMessages != null) {
			return (String)errorMessages.get(locale);
		} else {
			return null;
		}
	}

	public void setErrorMessage(String str, String locale) {
		if(errorMessages == null)
			errorMessages = new HashMap();
		errorMessages.put(locale, str);
	}
	
	//
	//
	//
	
	public String toXML(String indent) {
		return "\n" + indent + "<"+tagName()+">" + childrenToXML(indent+Form.PP_INDENT) + "\n" + indent + "</"+tagName()+">";
	}
	
	public String childrenToXML(String indent) {
		StringBuffer b = new StringBuffer();
		if(errorMessages != null) {
			Iterator it = errorMessages.keySet().iterator();
			while(it.hasNext()) {
				String loc = (String)it.next();
				if("".equals(loc)) {
					b.append("\n" + indent + "<"+Form.FBTAG_ERROR+">" + errorMessages.get(loc) + "</"+Form.FBTAG_ERROR+">");
				} else {
					b.append("\n" + indent + "<"+Form.FBTAG_ERROR+" "+Form.FBATTR_LOCALE+"=\"" +loc+ "\">" + errorMessages.get(loc) + "</"+Form.FBTAG_ERROR+">");
				}
			}
		}
 		if(b.length() == 0)
 			return null;
 		else
	 		return b.toString();
	}
}
