/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.forms;

import org.w3c.dom.*;


import java.util.regex.*;

/**
 * <p>&lt;email&gt;</p>
 * 
 * Matches value with email pattern and returns true or false depending on the outcome.<br> 
 * If value is null or empty string it evaluates to true without matching the pattern.<br>
 * 
 * Parameters:<br>
 *   error<br>
 *
 */
public class EmailValidator extends A_Validator {
	
	static private Pattern pattern = Pattern.compile("([a-zA-Z0-9_\\-\\.])+@((([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\\.([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5]))|((([a-zA-Z0-9\\-])+\\.)+([a-zA-Z\\-])+))");

	public EmailValidator(Form form) {
		super(form);
	}
	
	public EmailValidator(Form form, Node node) {
		super(form, node);
	}
	
	public boolean validate(I_Field field) {
		String value = field.getValue();
		if(value==null  ||value.length() == 0)
			return true;
		Matcher matcher = pattern.matcher(value);
		return matcher.matches();
	}

}
