/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import java.util.HashMap;
import java.util.Iterator;

import com.codedroids.j.StringUtils;
import java.util.Collections;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * Not really a value field, just a convenience for printing banners messages. 
 * Rendering should render label as usual and banner as if it where the value. Only thing is that
 * the banner text can be localized which the value cannot. If the banner text contains "${value}" then this
 * will be replaced by the value of &lt;value&gt;.
 * If &lt;label&gt; and /&lt;ps&gt; is not given, then the banner will strech all the way across the entire
 * row.
 *
 * &lt;message&gt;<br>
 * Children (besides the inherited):<br>
 *   banner<br>
 */
public class MessageField extends A_SingleValueField {

	private HashMap banners;
	
	public MessageField(Form form) {
		super(form);
	}

	public MessageField(Form form, Node field) {
		super(form, field);
	}

	public String tagName() {
		return Form.FBTAG_MESSAGE;
	}
	
	/**
	 * Replace ${value} entries in the text string with the value setting
	 */
	private static final int _len = "${value}".length();
	public String getBanner() {
		String retval = getBanner(form.getLocale());
		if(retval==null && banners!=null) 
			retval = (String)banners.get("");
		if(retval != null && retval.length()>=_len) // 8 is length of 
			retval = StringUtils.replaceMacroStrings(retval, Collections.singletonMap("value", getValue()));
		return retval;
	}
	
	public void setBanner(String str) {
		setBanner(str, "");
	}

	// Since this is primarily to use for generating xml output it does not alter the string
	public String getBanner(String locale) {
		if(banners != null) {
			return (String)banners.get(locale);
		} else {
			return null;
		}
	}

	public void setBanner(String str, String locale) {
		if(banners == null)
			banners = new HashMap();
		banners.put(locale, str);
	}

	//
	//
	//
	
	/**
	 * Look for relevant sub-elements
	 * @param child
	 * @param childType
	 * @param childName
	 * @return true if child is considered handled, false if it didn't cause any changes
	 */
	public boolean handleChildNode(Node child, short childType, String childName)
	{
		if(super.handleChildNode(child, childType, childName))
			return true;
		if(childType == Node.ELEMENT_NODE) {
			if(childName.equals(Form.FBTAG_BANNER)) {
				String v = Form.collectText(child);
				String l = ""; // in case it is not specified 
				NamedNodeMap attrs = child.getAttributes();
				for(int i=0; i<attrs.getLength(); i++) {
					Node attr = attrs.item(i);
					String an = attr.getNodeName();
					if(an.equals(Form.FBATTR_LOCALE)) {
						l = Form.collectText(attr);
					}
				}
				setBanner(v, l);
				return true;
			}
		}
		return false;
	}
	
	public String childrenToXML(String indent, boolean inclVal) 
	{
		String tmp = super.childrenToXML(indent, inclVal);
	    if(tmp == null && getValue() == null)
	    	return null;
		StringBuffer b = new StringBuffer();
		if(tmp != null)
			b.append(tmp);
		if(banners != null) {
			Iterator it = banners.keySet().iterator();
			while(it.hasNext()) {
				String loc = (String)it.next();
				if("".equals(loc)) {
					b.append("\n" + indent + "<"+Form.FBTAG_BANNER+">" + banners.get(loc) + "</"+Form.FBTAG_BANNER+">");
				} else {
					b.append("\n" + indent + "<"+Form.FBTAG_BANNER+" "+Form.FBATTR_LOCALE+"=\"" +loc+ "\">" + banners.get(loc) + "</"+Form.FBTAG_BANNER+">");
				}
			}
		}
 		return b.toString();
	}
}
