/*
 * Copyright (c) 2006-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.oc;

import org.opencms.jsp.CmsJspBean;
import org.opencms.file.CmsFile;
//import org.opencms.file.CmsResourceFilter;
import org.opencms.file.CmsResource;
import org.opencms.main.CmsException;

/**
 * Wraps a CmsFile object so it becomes cacheable. 
 *
 */
public class OCCachedFile implements I_OCCacheElement, I_OCCacheElementWrapper {
	
	/**
	 * <p>Convenience for caching a CmsFile object</p>
	 * @param cms
	 * @param path The VFS path (without site root)
	 * @return CmsFile object or null
	 * @throws CmsException
	 */
	public static CmsFile cacheFile(OCCache cache, CmsJspBean cms, String path) throws CmsException {
		OCCachedFile el = (OCCachedFile)cache.cacheElement(cms, cms.getRequestContext().addSiteRoot(path), OCCachedFile.class, path);
		return (el == null ? null : (CmsFile)el.getCacheValue());
	}
	
	/**
	 * <p>Convenience for looking up (and possible loading/caching) a CmsFile object</p>
	 * @param cms
	 * @param path The VFS path (without site root)
	 * @return CmsFile object or null
	 * @throws CmsException
	 */
	public static CmsFile getCachedFile(OCCache cache, CmsJspBean cms, String path) throws CmsException {
		OCCachedFile el = (OCCachedFile)cache.getCachedElement(cms, cms.getRequestContext().addSiteRoot(path), OCCachedFile.class, path);
		return (el == null ? null : (CmsFile)el.getCacheValue());
	}
	
	private CmsFile value;
	private String path;
	boolean online;
	
	public OCCachedFile() {
		super();
	}
	
	public boolean init(CmsJspBean cms, boolean online, String key, Object context) throws CmsException {
		this.online = online;
		this.path = (String)context;
		return init(cms);
	}
	
	private boolean init(CmsJspBean cms) throws CmsException {
		CmsResource res = cms.getCmsObject().readResource(path);
		if(res != null) {
			setCacheValue(CmsFile.upgrade(res,cms.getCmsObject()));
			return true;
		}
		return false;
	}
	
	public boolean refresh(CmsJspBean cms) throws CmsException {
		return init(cms);
	}

	/**
	 * Note that this will try to do the right thing as far as the expire date goes, but the nature of the offline cache's
	 * 'minimum cache period' causes changes to the release/expire dates to go undetected until the next time we check the modified
	 * date (which is changed and subsequently cause a refresh).
	 * 
	 * @param bestBefore
	 * @return
	 */
	public boolean isStale(CmsJspBean cms) throws CmsException {
		if(online) {
			// Online can only become stale because of the expire date (and after publish, but OCCache handles that case)
			return (value.getDateExpired() <= System.currentTimeMillis());
		} else {
			CmsResource res = cms.getCmsObject().readResource(path);
			if(res.getDateLastModified() <= value.getDateLastModified()) {
				return false;
			}
			return true;
		}
	}

	public void setCacheValue(Object value) {
		this.value = (CmsFile)value;
	}
	
	public Object getCacheValue() {
		return value;
	}

}
