/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.oc.jam;

import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import org.opencms.jsp.CmsJspActionElement;

import javax.xml.parsers.ParserConfigurationException;

import org.opencms.file.CmsObject;
import org.opencms.file.CmsResource;
import org.opencms.main.CmsException;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.codedroids.jam.JspApp;
import com.codedroids.oc.OCUtils;

public class OCJspApp extends JspApp {

	private static Map appsCache;
	private static String CACHE_TIMESTAMP = "timestamp";
	private static String CACHE_INSTANCE = "instance";

	static {
		appsCache = new HashMap();
	}
	
	/**
	 * Returns an instance, possible from the cache (if it is already loaded and file has not been modified since)
	 * 
	 * @param cmso
	 * @param xmlfile
	 * @param reload
	 * @return
	 */
	public static synchronized OCJspApp getInstance(CmsObject cmso, String xmlfile) throws ParserConfigurationException, SAXException, IOException, CmsException
	{
		// Currently caches the document tree rather than OCForm instances 
		// (which would have to be cloned as the instance is not immutable)
		Element root = null;
		Map m = (Map)appsCache.get(xmlfile);
		if(m != null) {
			long timestamp = ((Date)m.get(CACHE_TIMESTAMP)).getTime();
			CmsResource f = cmso.readResource(xmlfile);
			if(f != null && f.getDateLastModified() <= timestamp)
				root = (Element)m.get(CACHE_INSTANCE);
		}
		if(root == null) {
			root = OCUtils.loadXMLFileFromVFS(cmso, xmlfile).getDocumentElement();
			m = new HashMap();
			m.put(CACHE_INSTANCE, root);
			m.put(CACHE_TIMESTAMP, new Date());
			appsCache.put(xmlfile, m);
		}
		OCJspApp retval = new OCJspApp();
		retval.parseNode(root);
		return retval;
	}

	//
	// Convenience
	//
	
	/**
	 * Assumes that the surrounding folder is the application folder / bundle
	 */
	public static String appFolder(CmsJspActionElement cms) {
		return cms.info("opencms.request.folder");
	}
	
	public static String appName(CmsJspActionElement cms) {
		String folder = appFolder(cms);
		String name;
		int idx = folder.lastIndexOf("/");
		if(idx == folder.length()-1) {
			folder = folder.substring(0,idx);
			idx = folder.lastIndexOf("/");
		}
		if(idx!=-1) {
			name = folder.substring(idx+1);
		} else {
			name = folder;
		}
		idx = name.lastIndexOf(".");
		if(idx!=-1) {
			name = name.substring(0, idx);
		}
		name = cms.property("jam_name", "search", name);
		return name;
	}
	
	public static String appPrefix(CmsJspActionElement cms) {
		String folder = cms.info("opencms.request.folder");
		String name;
		String prefix;
		int idx = folder.lastIndexOf("/");
		if(idx == folder.length()-1) {
			folder = folder.substring(0,idx);
			idx = folder.lastIndexOf("/");
		}
		if(idx!=-1) {
			name = folder.substring(idx+1);
		} else {
			name = folder;
		}
		idx = name.lastIndexOf(".");
		if(idx==-1) {
			prefix = "jam:";
		} else {
			prefix = name.substring(idx+1) + ":";
		}
		prefix = cms.property("jam_prefix", "search", prefix);
		return prefix;
	}
}
