/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 150 $
 * $LastChangedDate: 2007-03-11 16:47:44 +0100 (Sun, 11 Mar 2007) $
 * 
 */

package com.codedroids.records;

import java.text.ParseException;
import java.lang.reflect.*;

public abstract class BeanRecord extends A_Record {

	static final long serialVersionUID =1L;
	
	public BeanRecord() {
		super();
	}

	public BeanRecord(RecordFactory factory) {
		super(factory);
	}

	public BeanRecord(RecordFactory factory, String id) throws ParseException, RecordException {
		super(factory, id);
	}

	public BeanRecord(RecordFactory factory, Object id) throws RecordException {
		super(factory, id);
	}

	public void takeValueForField(RecordField field, Object value) {
		String methodName = "set" + field.getName().substring(0,1).toUpperCase() + field.getName().substring(1);
		try {
			Method m = this.getClass().getMethod(methodName, new Class[] { field.fieldClass() } );
			m.invoke(this, new Object[] { value });		
		} catch (Exception e) {
			throw new IllegalArgumentException(getClass().getName() + " takeValueForField(): field=\""+field.getName()+"\" threw: "+e);
			//e.printStackTrace();
		}
	}
	
	public Object valueForField(RecordField field) {
		String methodName = "get" + field.getName().substring(0,1).toUpperCase() + field.getName().substring(1);
		try {
			Method m = this.getClass().getMethod(methodName, null );
			return m.invoke(this, null);			
		} catch (Exception e) {
			throw new IllegalArgumentException(getClass().getName() + " valueForField(): field=\""+field.getName()+"\" threw: "+e);
		}
	}
}
