/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 150 $
 * $LastChangedDate: 2007-03-11 16:47:44 +0100 (Sun, 11 Mar 2007) $
 * 
 */

package com.codedroids.records;

import java.text.ParseException;

public interface I_Record {

	/**
	 * Provides the actual class initialization
	 * @param factory
	 */
	public void init(RecordFactory factory);
	public void init(RecordFactory factory, Object id) throws RecordException;
	
    /**
     * A record must now its origin
     * @return
     */
	public RecordFactory factory();
	
	/**
	 * If record has no representation in the database this must return true.
	 * If it has been saved or it is fetched from the database this returns false.
	 * @return
	 */
	public boolean isNew();
	
	/**
	 * This tells the record that is has been saved to the database.
	 */
	public void saved();
	
	/**
	 * This tells the record that it has been initialized with values fetched from the database.
	 */
	public void fetched();
	
	/**
	 * Looks up the particular field name (maybe by interrogating the factory). Returns true if
	 * the field is among those known to the factory (i.e. fields that persist). 
	 * @param fieldName
	 * @return
	 */
	public boolean hasField(String fieldName);
	
	//
	// KVEncoding, primitives are wrapped.
	//
	
	/**
	 * sets and gets value for the given field. The value must be of the correct type for the field.
	 */
	public void takeValueForField(RecordField field, Object value);
	public Object valueForField(RecordField field);
	
	/**
	 * Convenience - does a lookup for the field. If field is unknown it should throw an exception.
	 * @param key
	 * @param value
	 */
	public void takeValueForKey(String key, Object value);
	public Object valueForKey(String key);
	
	/**
	 * Similar methods, only they will attempt to convert the given string value into a value with
	 * the value type for the field. 
	 * @param key
	 * @param value
	 * @throws ParseException
	 */
	public void takeStringValueForKey(String key, String value) throws ParseException;
	public void takeStringValueForField(RecordField field, String value) throws ParseException;
	public String stringValueForKey(String key);
	public String stringValueForField(RecordField field); 
	
	/**
	 * These serves as convenience for handling primitive values, throws if there are class casting problems
	 */
	public boolean booleanValueForKey(String key);
	public int intValueForKey(String key);
}
